{ nixpkgs ? import <nixpkgs> {}, compiler ? "default", doBenchmark ? false }:

let

  inherit (nixpkgs) pkgs;

  affection = with haskellPackages; callPackage(
    { mkDerivation, base, bytestring, clock, containers, glib, linear
    , monad-loops, monad-parallel, mtl, OpenGL, sdl2, stdenv, stm, text
    , uuid, vector
    }:
    mkDerivation {
      pname = "affection";
      version = "0.0.0.10";
      src = ./extern/affection;
      configureFlags = [ "-fverbose" ];
      isLibrary = true;
      isExecutable = true;
      libraryHaskellDepends = [
        base bytestring clock containers glib linear monad-loops
        monad-parallel mtl OpenGL sdl2 stm text uuid vector
      ];
      homepage = "https://github.com/nek0/affection#readme";
      description = "A simple Game Engine using SDL";
      license = pkgs.lib.licenses.lgpl3;
    }) {};

  f = { mkDerivation, aeson, base, bytestring, containers, derive-storable
      , JuicyPixels, JuicyPixels-extra, linear, monad-loops, OpenGL, OpenGLRaw
      , stdenv, sdl2, StateVar, stm, text, vector}:
      mkDerivation {
        pname = "pituicat";
        version = "0.0.0.0";
        src = ./.;
        isLibrary = false;
        isExecutable = true;
        executableHaskellDepends = [ aeson affection base bytestring containers
        derive-storable JuicyPixels JuicyPixels-extra linear monad-loops OpenGL
        OpenGLRaw sdl2 StateVar stm text vector];
        license = pkgs.lib.licenses.gpl3;
      };

  haskellPackages = if compiler == "default"
                       then pkgs.haskellPackages
                       else pkgs.haskell.packages.${compiler};

  variant = if doBenchmark then pkgs.haskell.lib.doBenchmark else pkgs.lib.id;

  drv = variant (haskellPackages.callPackage f {});

in

  if pkgs.lib.inNixShell then drv.env else drv
